package com.seeyon.apps.govdoc.controller;

import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.web.servlet.ModelAndView;

import com.seeyon.apps.govdoc.resourceManager.GovdocResourceEnum.ResourceMemberTypeEnums;
import com.seeyon.ctp.common.AppContext;
import com.seeyon.ctp.common.controller.BaseController;
import com.seeyon.ctp.organization.OrgConstants.Role_NAME;
import com.seeyon.ctp.resourceEngine.bo.ResourceItem;
import com.seeyon.ctp.resourceEngine.bo.ResourceItemValueBO;
import com.seeyon.ctp.resourceEngine.bo.ResourceValueBO;
import com.seeyon.ctp.resourceEngine.dao.PrivResourceConfigDAO;
import com.seeyon.ctp.resourceEngine.enums.ResourceConfigEnums.ResourceExpressionEnums;
import com.seeyon.ctp.resourceEngine.enums.ResourceConfigEnums.ResourceInitFlagEnums;
import com.seeyon.ctp.resourceEngine.enums.ResourceConfigEnums.ResourceSelectTypeEnums;
import com.seeyon.ctp.resourceEngine.enums.ResourceConfigEnums.ResourceTypeEnums;
import com.seeyon.ctp.resourceEngine.manager.PrivilegeConfigManager;
import com.seeyon.ctp.resourceEngine.manager.ResourceService;
import com.seeyon.ctp.resourceEngine.po.PrivResourceConfig;
import com.seeyon.ctp.util.DateUtil;
import com.seeyon.ctp.util.ParamUtil;
import com.seeyon.ctp.util.Strings;
import com.seeyon.ctp.util.annotation.CheckRoleAccess;
import com.seeyon.ctp.util.json.JSONUtil;

@CheckRoleAccess(roleTypes={Role_NAME.GroupAdmin,Role_NAME.AccountAdministrator})
public class GovdocResourceController extends BaseController {
	
	private PrivilegeConfigManager privilegeConfigManager;
	private PrivResourceConfigDAO privResourceConfigDAO;
	
	public ModelAndView saveResourceConfig(HttpServletRequest request, HttpServletResponse response) throws Exception {
		Map<String,String> baseInfo =ParamUtil.getJsonDomain("baseInfo");
		PrivResourceConfig prc = null;
		String resourceId = (String) baseInfo.get("resourceId");
		if(Strings.isBlank(resourceId)){
			prc = new PrivResourceConfig();
			prc.setIdIfNew();
			prc.setCreatedate(DateUtil.currentTimestamp());
			prc.setCreateuserid(AppContext.getCurrentUser().getId());
		}else{
			prc = privResourceConfigDAO.findById(Long.valueOf(resourceId));
		}
		prc.setCode(baseInfo.get("code"));
		prc.setAccountId(AppContext.currentAccountId());
		prc.setDesc(baseInfo.get("desc"));
		prc.setInitFlag(ResourceInitFlagEnums.CustomList.getKey());
		prc.setName(baseInfo.get("resourceName"));
		prc.setPlugin(baseInfo.get("plugin"));
		prc.setSortid(0);
		prc.setType(ResourceTypeEnums.customResource.getKey());
		prc.setUpdatedate(DateUtil.currentTimestamp());
		prc.setUpdateuserid(AppContext.getCurrentUser().getId());
		prc.setCopyFrom(Strings.isBlank(baseInfo.get("copyFrom"))?0l:Long.parseLong(baseInfo.get("copyFrom")));
		
		ResourceValueBO bo = new ResourceValueBO(prc,false);
		
		//统计范围
		Map<String,String> domain =ParamUtil.getJsonDomain("ds_member");
		String affairMember = domain.get("affairMember");
		if(!ResourceMemberTypeEnums.currentMember.name().equals(affairMember) && !ResourceMemberTypeEnums.currentDepartment.name().equals(affairMember)){
			affairMember = domain.get("selectScope");
		}
		bo.getDataSources().add(new ResourceItemValueBO("affairMemberId",ResourceExpressionEnums.eq.getKey(),affairMember,null));
		
		//模块
		domain =ParamUtil.getJsonDomain("ds_subapp");
		for(int i=1;i<5;i++){
			String sa = domain.get("affairSubApp_"+i);
			if(Strings.isNotBlank(sa)){
				ResourceItemValueBO ibo = new ResourceItemValueBO("affairSubApp",ResourceExpressionEnums.in.getKey(),sa,null);
				sa = domain.get("affairNodePolicy_"+i);
				if(Strings.isNotBlank(sa)){
					ibo.getSubItem().add(new ResourceItemValueBO("affairNodePolicy",ResourceExpressionEnums.in.getKey(),sa,null));
				}
				bo.getDataSources().add(ibo);
			}
		}
		
		//节点状态
		domain =ParamUtil.getJsonDomain("ds_state");
		for(int i=1;i<7;i++){// 1-7的状态
			String state = domain.get("affairState_"+i);
			if(Strings.isNotBlank(state)){
				ResourceItemValueBO ibo = new ResourceItemValueBO("affairState",ResourceExpressionEnums.eq.getKey(),state,null);
				boolean isAll = false;//是否选了全部
				for(int k=0;k<4;k++){
					String substate = domain.get("affairSubState_"+i+"_"+k);
					if(k==0 && Strings.isNotBlank(substate)){
						isAll = true;
						break;
					}
					if(Strings.isNotBlank(substate)){
						ibo.getSubItem().add(new ResourceItemValueBO("affairSubState",ResourceExpressionEnums.eq.getKey(),substate,null));
					} 
				}
				if(i==4){//已办比较特别
					if(isAll || ibo.getSubItem().size() > 0){
						bo.getDataSources().add(ibo);
					}
				}else{
					bo.getDataSources().add(ibo);
				}
			}
		}
		
		//流程状态
		domain = ParamUtil.getJsonDomain("ds_summarystate");
		String sValue = "";
		for (Object entryObj : domain.entrySet()) {
			if (entryObj instanceof Map.Entry) {
				Map.Entry entry = (Map.Entry) entryObj;
				if(entry.getValue()!=null){
					sValue = sValue + "," + entry.getValue().toString();
				}
			}
		}
		if(Strings.isNotBlank(sValue)){
			ResourceItemValueBO ibo = new ResourceItemValueBO("summaryState",ResourceExpressionEnums.in.getKey(),sValue.substring(1),null);
			bo.getDataSources().add(ibo);
		}
		
		//公文属性
		List<Map> list = ParamUtil.getJsonDomainGroup("ds_attrs");
		if(list==null ||list.size() == 0){
			domain = ParamUtil.getJsonDomain("ds_attrs");
			String itemCode = String.valueOf(domain.get("field"));
			String itemValue = String.valueOf(domain.get("fieldValue"));
			ResourceItem ri = ResourceService.getInstance().getItemByHandlerCodAndItemCode(prc.getCode(), itemCode);
			String op = ResourceExpressionEnums.eq.getKey();
			if(ri != null && Strings.isNotBlank(itemValue)){
				if(ResourceSelectTypeEnums.input.getKey().equals(ri.getSelectType())){
					op = ResourceExpressionEnums.like.getKey();
				}
				if("turnRecUserId".equals(itemCode) && "1".equals(itemValue)){//后台 is not null
					op = ResourceExpressionEnums.ne.getKey();
				}
				bo.getDataSources().add(new ResourceItemValueBO(itemCode,op,itemValue,null));
			}
		}else{
			for(Map domain1:list){
				String itemCode = String.valueOf(domain1.get("field"));
				String itemValue = String.valueOf(domain1.get("fieldValue"));
				ResourceItem ri = ResourceService.getInstance().getItemByHandlerCodAndItemCode(prc.getCode(), itemCode);
				String op = ResourceExpressionEnums.eq.getKey();
				if(ri != null && Strings.isNotBlank(itemValue)){
					if(ResourceSelectTypeEnums.input.getKey().equals(ri.getSelectType())){
						op = ResourceExpressionEnums.like.getKey();
					}
					if("turnRecUserId".equals(itemCode) && "1".equals(itemValue)){//后台 is not null
						op = ResourceExpressionEnums.ne.getKey();
					}
					bo.getDataSources().add(new ResourceItemValueBO(itemCode,op,itemValue,null));
				}
			}
		}
		
		//公文元素
		list = ParamUtil.getJsonDomainGroup("ds_elements");
		if(list==null ||list.size() == 0){
			domain = ParamUtil.getJsonDomain("ds_elements");
			String itemCode = String.valueOf(domain.get("field"));
			String itemValue = String.valueOf(domain.get("fieldValue"));
			ResourceItem ri = ResourceService.getInstance().getItemByHandlerCodAndItemCode(prc.getCode(), itemCode);
			String op = ResourceExpressionEnums.eq.getKey();
			if(ri != null && Strings.isNotBlank(itemValue)){
				if(ResourceSelectTypeEnums.input.getKey().equals(ri.getSelectType())){
					op = ResourceExpressionEnums.like.getKey();
				}
				bo.getDataSources().add(new ResourceItemValueBO(itemCode,op,itemValue,null));
			}
		}else{
			for(Map domain1:list){
				String itemCode = String.valueOf(domain1.get("field"));
				String itemValue = String.valueOf(domain1.get("fieldValue"));
				ResourceItem ri = ResourceService.getInstance().getItemByHandlerCodAndItemCode(prc.getCode(), itemCode);
				String op = ResourceExpressionEnums.eq.getKey();
				if(ri != null && Strings.isNotBlank(itemValue)){
					if(ResourceSelectTypeEnums.input.getKey().equals(ri.getSelectType())){
						op = ResourceExpressionEnums.like.getKey();
					}
					bo.getDataSources().add(new ResourceItemValueBO(itemCode,op,itemValue,null));
				}
			}
		}
		
		//其他显示 
		domain = ParamUtil.getJsonDomain("ds_other");
		String showItemsJson = domain.get("showItemsJson");
		String seachItemsJson = domain.get("seachItemsJson");
		String toolbarItemsJson = domain.get("toolbarItemsJson");
		
		if(Strings.isNotBlank(showItemsJson)){
			list = JSONUtil.parseJSONString(showItemsJson,List.class);
			for(Map m:list){
				String code = String.valueOf(m.get("code"));
				String itemShowName = String.valueOf(m.get("itemShowName"));
				bo.getDataShowColumns().add(new ResourceItemValueBO(code,"","",itemShowName));
			}
		}
		if(Strings.isNotBlank(seachItemsJson)){
			list = JSONUtil.parseJSONString(seachItemsJson,List.class);
			for(Map m:list){
				String code = String.valueOf(m.get("code"));
				String itemShowName = String.valueOf(m.get("itemShowName"));
				bo.getDataSearchColumns().add(new ResourceItemValueBO(code,"","",itemShowName));
			}
		}
		if(Strings.isNotBlank(toolbarItemsJson)){
			list = JSONUtil.parseJSONString(toolbarItemsJson,List.class);
			for(Map m:list){
				String code = String.valueOf(m.get("code"));
				String itemShowName = String.valueOf(m.get("itemShowName"));
				bo.getDataToolBars().add(new ResourceItemValueBO(code,"","",itemShowName));
			}
		}
		privilegeConfigManager.saveOrUpdateResourceConfig(bo);
		return this.redirectModelAndView("/resourceEngine/resourceConfig.do?method=resourceDetail&viewState=view&modify=1&resourceId="+bo.getPrivResourceConfig().getId());
	}
	
    /**
     * 资源节点权限选择
     * @param request
     * @param response
     * @return
     * @throws Exception
     */
	public ModelAndView resourceNodeSelect(HttpServletRequest request, HttpServletResponse response) throws Exception {
        ModelAndView mav = new ModelAndView("apps/resource/govdocResource/resourceNodeSelect");
        return mav;
    }
	
	/**
	 * 数据源描述
	 * @param request
	 * @param response
	 * @return
	 * @throws Exception
	 */
	public ModelAndView resourceExplain(HttpServletRequest request, HttpServletResponse response) throws Exception {
        ModelAndView mav = new ModelAndView("apps/resource/govdocResource/resourceExplain_read");
        return mav;
    }

	public PrivilegeConfigManager getPrivilegeConfigManager() {
		return privilegeConfigManager;
	}

	public void setPrivilegeConfigManager(PrivilegeConfigManager privilegeConfigManager) {
		this.privilegeConfigManager = privilegeConfigManager;
	}

	public PrivResourceConfigDAO getPrivResourceConfigDAO() {
		return privResourceConfigDAO;
	}

	public void setPrivResourceConfigDAO(PrivResourceConfigDAO privResourceConfigDAO) {
		this.privResourceConfigDAO = privResourceConfigDAO;
	}

}
